/**
 * @module DatePicker Component
 */

import React, { Component, PropTypes } from 'react';
import DatePickerItem from './DatePickerItem.js';
import PureRender from './pureRender.js';
import { convertDate, nextDate } from './time.js';

/**
 * Class DatePicker Component Class
 * @extends Component
 */
class DatePicker extends Component {
    constructor(props) {
        super(props);
        this.state = {
            value: nextDate(this.props.value),
        };

        this.handleFinishBtnClick = this.handleFinishBtnClick.bind(this);
        this.handleDateSelect = this.handleDateSelect.bind(this);
    }

    componentWillReceiveProps(nextProps) {
        // update value of state
        const date = nextDate(nextProps.value);
        if (date.getTime() !== this.state.value.getTime()) {
            this.setState({ value: date });
        }
    }

    /**
     * Optimization component, Prevents unnecessary rendering
     * Only props or state change or value before re-rendering
     *
     * @param  {Object} nextProps next props
     * @param  {Object} nextState next state
     * @return {Boolean}          Whether re-rendering
     */
    shouldComponentUpdate(nextProps, nextState) {
        const date = nextDate(nextState.value);
        return date.getTime() !== this.state.value.getTime() ||
                PureRender.shouldComponentUpdate(nextProps, nextState, this.props, this.state);
    }

    /**
     * 点击完成按钮事件
     * @return {undefined}
     */
    handleFinishBtnClick() {
        this.props.onSelect(this.state.value);
    }

    /**
     * 选择下一个日期
     * @return {undefined}
     */
    handleDateSelect(value) {
        this.setState({ value });
        if(!this.props.showButton) {
            this.props.onSelect(this.state.value);
        }

    }

    /**
     * render函数
     * @return {Object} JSX对象
     */
    render() {
        const { min, max, dateFormat, loopAdd, showButton, type } = this.props;
        const value = this.state.value;

        return (
            <div
                className="datepicker">
                <div className="datepicker-content">
                    <DatePickerItem
                        value={value}
                        min={min}
                        max={max}
                        typeName="Year"
                        loopAdd={loopAdd}
                        format={dateFormat[0]}
                        type={type}
                        onSelect={this.handleDateSelect} />
                    <DatePickerItem
                        value={value}
                        min={min}
                        max={max}
                        typeName="Month"
                        loopAdd={loopAdd}
                        format={dateFormat[1]}
                        type={type}
                        onSelect={this.handleDateSelect} />
                    <DatePickerItem
                        value={value}
                        min={min}
                        max={max}
                        typeName="Date"
                        loopAdd={loopAdd}
                        format={dateFormat[2]}
                        type={type}
                        onSelect={this.handleDateSelect} />
                </div>
                {
                    showButton ?
                        <div className="datepicker-navbar">
                            <a
                              className="datepicker-navbar-btn"
                              onClick={this.handleFinishBtnClick}>完成</a>
                            <a
                              className="datepicker-navbar-btn"
                              onClick={this.props.onCancel}>取消</a>
                        </div> : null
                }
            </div>
        );
    }
 }

DatePicker.propTypes = {
    value: PropTypes.object,
    min: PropTypes.object,
    max: PropTypes.object,
    loopAdd: PropTypes.bool,
    showButton: PropTypes.bool,
    dateFormat: PropTypes.array,
    onSelect: PropTypes.func,
    onCancel: PropTypes.func,
};

export default DatePicker;
