import React, { Component, PropTypes } from 'react';
import classnames from 'classnames';
import { render, unmountComponentAtNode } from 'react-dom';

const prefix = 'Modal--';
let instances = [];

export default class Modal extends Component {

    static propTypes = {
        className: PropTypes.string,
        containerClassName: PropTypes.string,
        animation: PropTypes.string,
        disableAnimation: PropTypes.bool,
        type: PropTypes.oneOf(['default', 'small']),
        title: PropTypes.string,
        okText: PropTypes.string,
        cancelText: PropTypes.string,
        onClose: PropTypes.func,
        footer: PropTypes.oneOfType([
            PropTypes.node,
            PropTypes.element,
        ]),
        header: PropTypes.oneOfType([
            PropTypes.node,
            PropTypes.element,
        ]),
        backdropCloseable: PropTypes.bool,
        backdropMoveable: PropTypes.bool,
    }

    static defaultProps = {
        animation: 'scaledown',
        backdropMoveable: false,
        backdropCloseable: false,
        disableAnimation: false,
        type: 'default',
        containerClassName: 'Modal__container',
        contentClassName: 'Modal__content'
    }

    constructor(args) {
        super(args);

        document.addEventListener('touchmove', this.globalTouchMoveHandler);
    }

    componentWillUnmount() {
        document.removeEventListener('touchmove', this.globalTouchMoveHandler);
    }

    globalTouchMoveHandler(e) {
        const target = e.target;

        // outside touch move should be prevented
        if((target.classList.contains('.Modal__container') ||
              target.querySelector('.Modal__container')) &&
            !this.backdropMoveable) {
            e.preventDefault();
        }

        e.stopPropagation();
    }

    onClose(isOk) {
        if(this.props.onClose) {
            this.props.onClose(isOk);
        } else {
            Modal.close();
        }
    }

    onBackdropClick = (e) => {
        if(this.props.backdropCloseable &&
            e.target === this.refs.modal) {
            this.onClose();
        }
    }

    render() {
        let {
            className,
            containerClassName,
            children,
            type,
            title,
            header,
            footer,
            okText,
            cancelText,
            animation,
            disableAnimation,
            ...others
        } = this.props;

        animation = type === 'small' ? animation : 'slidedown';

        const classes = classnames({
            'Modal_new': true,
            [className]: className,
            'Modal--small': type === 'small'
        });

        const containerClass = classnames({
            [containerClassName]: true,
            [`anim-${animation}`]: !disableAnimation,
            [containerClassName]: containerClassName,
        });

        let modalCount = instances.length;

        return (
            <div
                {...others}
                className = {classes}
                style={{zIndex: 5000 + modalCount * 100}}
                ref="modal"
                onClick={this.onBackdropClick}>
                <style>{'html {overflow: hidden;}'}</style>
                <div className={containerClass} autoFocus>
                    {
                        title ?
                            <div className="Modal__header">
                                <h3> { title } </h3>
                                { type !== 'small' &&
                                    <span className="Modal_new__close"
                                      title="关闭"
                                      onClick={this.onClose.bind(this, false)}>
                                       &times;
                                    </span>
                                }
                            </div> : header
                    }
                    <div className="Modal__content" ref="modalContent">
                        {
                            (typeof children === 'string') ?
                                <div className="Modal__text">{ children } </div> :
                                children
                        }
                    </div>
                    {
                        footer === undefined ?
                            <div className="Modal__footer">
                                { !!cancelText &&
                                    <span className="Modal__btn"
                                      onClick={this.onClose.bind(this, false)}>
                                    { cancelText }
                                    </span>
                                }
                                { !!okText &&
                                    <span className="Modal__btn"
                                        onClick={this.onClose.bind(this, true)}>
                                        { okText }
                                    </span>
                                }
                            </div> : footer
                    }
                </div>
            </div>
        )
    }

}

Modal.open = Modal.show = (modal, id, keepAlive) => {
    let target = document.createElement('div');
    keepAlive = typeof id === 'boolean' ? id : keepAlive;
    id = typeof id === 'boolean' ? Date.now() : (id || Date.now());
    let target_id = `vu-modal-${id}`;
    instances.push({id: id, keepAlive: keepAlive});
    target.id = target_id;
    document.body.appendChild(target);
    if(!React.isValidElement(modal)) {
        modal = createModal(modal)
    }
    render(modal, target, () => {
        if(modal.props.disableAnimation) {
          document.body.classList.add('Modal--open');
        } else {
          setTimeout(() => document.body.classList.add('Modal--open'), 10);
        }
    });
      console.dir(instances)
};

function createModal(props) {
    return (
        <Modal {...props}/>
    )
}

Modal.close = Modal.hide = (id) => {
    if (instances.length > 0 ) {
        if (id) {
            let i = -1;
            instances.forEach((item, index) => {
                if(item.id === id) i = index;
            })

            if(i !== -1) {
                instances.splice(i, 1);
            } else {
                console.log('id isn\'t right');
                return;
            }
        }

        id = id || instances.pop().id;

        if (instances.length === 0) {
            document.body.classList && document.body.classList.remove('Modal--open');
        }

        let target_id = `vu-modal-${id}`;
        if (target_id) {
            setTimeout(() => {
                let target = document.getElementById(target_id);
                target && window.requestAnimationFrame(() => {
                    unmountComponentAtNode(target);
                    document.body.removeChild(target);
                });
            }, 300);
        }
    }
};

Modal.closeAll = () => {
    instances.forEach(item => {
        if(!item.keepAlive){
            Modal.close(item.id);
        }
    })
}

Modal.showAndHide=(modal,id,time)=>{
    Modal.show(modal,id);
    setTimeout(()=>{Modal.hide(id)},time);
};
