import React from 'react';
import ReactDOM from 'react-dom';
import { Loading } from 'vision-ui'

let positionCache = {};

export default class ScrollPaging extends React.Component {

    constructor(props) {
        super(props);
        this.locationHash = location.hash;
        this.scrollFunction = this.scrollListener.bind(this);
    }

    componentDidMount () {
        const el = this._findElement();
        const name = this.locationHash;

        el.scrollTop = 0;   // 为了避免即将上映未滚动情况下受到正在上映scrollTop影响
        
        if (positionCache[name] && this.props.retainPosition) {
            el.scrollTop = positionCache[name];
        } 

        this.attachScrollListener();
    }

    componentDidUpdate (prevProps) {
        const el = this._findElement();
        const name = this.locationHash;

        this.attachScrollListener(prevProps);
    }

    componentWillUnmount () {
        this.detachScrollListener();
        this.retainPagePostion();
    }

    _findElement() {
        return this.props.parentNode || ReactDOM.findDOMNode(this).parentNode
    }

    attachScrollListener (prevProps) {
        if (!this.props.hasMore || this.props.loadingMore) return;
        let el = this._findElement();
        el.addEventListener('scroll', this.scrollFunction, true);
        el.addEventListener('resize', this.scrollFunction, true);
        this.scrollListener();
    }

    retainPagePostion() {
        const el = this._findElement();
        const name = this.locationHash;

        positionCache[name] = el.scrollTop;
    }

    _elScrollListener() {
        let el = this._findElement();
        let topScrollPos = el.scrollTop;
        let totalContainerHeight = el.scrollHeight;
        let containerFixedHeight = el.offsetHeight;
        let bottomScrollPos = topScrollPos + containerFixedHeight;

        return (totalContainerHeight - bottomScrollPos);
    }

    scrollListener() {
        // This is to prevent the upcoming logic from toggling a load more before
        // any data has been passed to the component
        if (this._totalItemsSize() <= 0) return;

        let bottomPosition = this._elScrollListener();

        if (this.props.auto && (bottomPosition < Number(this.props.threshold))) {
            this.detachScrollListener();
            this.props.loadMore();
        }
    }

    detachScrollListener () {
        let el = this._findElement();
        el.removeEventListener('scroll', this.scrollFunction, true);
        el.removeEventListener('resize', this.scrollFunction, true);
    }

    renderItems() {
        const allItems = this.props.children.concat(this.props.items);
        return allItems;
    }

    renderAddition() {
        return this.props.additionTemplate;
    }

    _totalItemsSize() {
        let totalSize;
        totalSize += this.props.children.length;
        totalSize += this.props.items.length;
        return totalSize;
    }

    renderLoader() {
        return (this.props.hasMore && this.props.showLoader) ? this.props.loader : undefined;
    }

    _assignHolderClass() {
        let additionalClass;
        additionalClass = (typeof this.props.className === 'function') ? this.props.className() : this.props.className;

        return 'scroll-paging ' + additionalClass;
    }

    loadManually() {
        this.props.loadMore()
    }

    render() {
        const Holder = this.props.holderType;

        return (
            <Holder className={ this._assignHolderClass() }>
                {this.renderItems()}
                <div onClick={!this.props.auto && this.loadManually.bind(this)}>
                {this.renderLoader()}
                </div>
                {!this.props.hasMore ? this.renderAddition() : null}
            </Holder>
        )
    }
}

ScrollPaging.propTypes = {
    additionTemplate: React.PropTypes.any,
    threshold: React.PropTypes.number,
    hasMore: React.PropTypes.bool,
    loadingMore: React.PropTypes.bool,
    auto: React.PropTypes.bool,
    loader: React.PropTypes.any,
    showLoader: React.PropTypes.bool,
    retainPosition: React.PropTypes.bool,
    loadMore: React.PropTypes.func.isRequired,
    items: React.PropTypes.oneOfType([
            React.PropTypes.array
    ]),
    children: React.PropTypes.oneOfType([
            React.PropTypes.array
    ]),
    holderType: React.PropTypes.string,
    className: React.PropTypes.oneOfType([
            React.PropTypes.string,
            React.PropTypes.func
    ])
};

ScrollPaging.defaultProps = {
    additionTemplate: null, // 没有更多时的展示模板
    auto: true, // 是否自动加载更多
    className: '',
    threshold: 100, // 单位px，距离底部多少时会触发loading
    hasMore: true,  // 数据是否还有更多可以加载
    loadingMore: false, // 正在加载更多
    loader: <div className="loading-more"><Loading type="circle" /></div>, //加载中的模板
    showLoader: true, // 是否展示loader
    retainPosition: false, //返回上一页面回到原来位置
    holderType: 'div',
    children: [],
    items: []
};
